#include <stdio.h>
#include <string.h>
#include "nrf_drv_timer.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_pwm.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "boards.h"
#include "bsp.h"
#include "nrf_drv_clock.h"
#include "nrf_delay.h"
#include "nrf_drv_gpiote.h"
#include "nrfx_timer.h"




#define OUTPUT_PIN LED_1
#define OUTPUT_PIN2 LED_2

#define    PIN_IN    22
#define    PWM_PIN   11

#define PWM_DELAY_US   1000 


unsigned int duty = 10000 ;
static int  gpio_flag;
  
void pwm_update_duty_cycle(uint16_t duty_cycle);

static nrf_drv_pwm_t m_pwm0 = NRF_DRV_PWM_INSTANCE(0);
const nrfx_timer_t PWM_DELAY_TIMER = NRFX_TIMER_INSTANCE(1);
static nrf_ppi_channel_t ppi_channel; // A struct to hol ppi channel values

uint32_t compare_event_addr;
uint32_t pwm_task_addr;



// Declare variables holding PWM sequence values. In this example only one channel is used 
nrf_pwm_values_individual_t seq_values[] = {0, 0, 0, 0};

nrf_pwm_sequence_t const seq =
{
    .values.p_individual = seq_values,
    .length          = NRF_PWM_VALUES_LENGTH(seq_values),
    .repeats         = 0,
    .end_delay       = 0
};

void pwm_delay_timeout_handler(nrf_timer_event_t event_type, void* p_context)
{
    UNUSED_PARAMETER(p_context);
    pwm_update_duty_cycle(duty);

    //nrf_drv_gpiote_in_event_disable(PIN_IN);
    //nrf_drv_ppi_channel_disable(ppi_channel);
    
    
}

void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
   
    
}

static void gpio_init(void)
{
    ret_code_t err_code;
    

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

    // Initialize the PPI module, make sure its only enabled once in your code
    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);

// Allocate the channel from the available PPI channels
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);
    in_config.pull = NRF_GPIO_PIN_PULLDOWN;

    err_code = nrf_drv_gpiote_in_init(PIN_IN, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);



    nrf_drv_gpiote_in_event_enable(PIN_IN, true);
}

void timers_init(void)
{
    uint32_t err_code;
    uint32_t timer_ticks;
    
    nrfx_timer_config_t timer_cfg = NRFX_TIMER_DEFAULT_CONFIG;
    timer_cfg.frequency = NRF_TIMER_FREQ_1MHz;
    err_code = nrfx_timer_init(&PWM_DELAY_TIMER, &timer_cfg,pwm_delay_timeout_handler);
    APP_ERROR_CHECK(err_code);

    timer_ticks = nrfx_timer_us_to_ticks(&PWM_DELAY_TIMER, PWM_DELAY_US);

    nrfx_timer_extended_compare(&PWM_DELAY_TIMER, 
                                  NRF_TIMER_CC_CHANNEL0, 
                                  timer_ticks,
                                  NRF_TIMER_SHORT_COMPARE0_STOP_MASK,
                                  true);

   nrfx_timer_enable(&PWM_DELAY_TIMER);

    compare_event_addr = nrf_drv_gpiote_in_event_addr_get(PIN_IN);
    pwm_task_addr =  nrf_drv_timer_task_address_get(&PWM_DELAY_TIMER, NRF_TIMER_TASK_START);

    err_code = nrf_drv_ppi_channel_assign(ppi_channel, compare_event_addr, pwm_task_addr);
    APP_ERROR_CHECK(err_code);


    err_code = nrf_drv_ppi_channel_enable(ppi_channel);
    APP_ERROR_CHECK(err_code);

}


// Set duty cycle between 0 and 100%
 
 void pwm_update_duty_cycle(uint16_t duty_cycle)
{
  
        seq_values->channel_0 = duty_cycle ;
        seq_values->channel_1 = duty_cycle | 0x8000;

        nrf_drv_pwm_simple_playback(&m_pwm0, &seq, 1, NRF_DRV_PWM_FLAG_LOOP);
    
     
}

static void pwm_init(void)
{
    nrf_drv_pwm_config_t const config0 =
    {
        .output_pins =
        {
            OUTPUT_PIN, // channel 0
            OUTPUT_PIN2,             // channel 1
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 2
            NRF_DRV_PWM_PIN_NOT_USED,             // channel 3
        },
        .irq_priority = APP_IRQ_PRIORITY_LOWEST,
        .base_clock   = NRF_PWM_CLK_1MHz,
        .count_mode   = NRF_PWM_MODE_UP,
        .top_value    = 20000,
        .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
        .step_mode    = NRF_PWM_STEP_AUTO
    };
    // Init PWM without error handler
    APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm0, &config0, NULL));
    
}




int main(void)
{

  // Start accurate HFCLK (XOSC)
  NRF_CLOCK->TASKS_HFCLKSTART = 1;
  while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) ;
  NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
   
    bsp_board_init(BSP_INIT_LEDS);
    gpio_init();
    timers_init();
    pwm_init();
    
     
    unsigned int val;
    
    while (1)
    {
      
      
       
    }
}
